PairRev

Struct PairRev 

pub struct PairRev<A, B>(pub B, pub A);
Sequential composition of formats A and B, where the direction of parsing is reversed compared to super::Pair.

Parsing semantics: parses B from the back, consumes the tail part of the input, then parses A.

Tuple Fields

0: B1: A